"use strict";

/*
This module is derived from: css-loader (https://github.com/webpack-contrib/css-loader)
	(c) JS Foundation and other contributors

Modifications by:
	(c) 2019, Sven Slootweg <admin@cryto.net>

Licensed under:
	MIT (https://opensource.org/licenses/MIT)
*/

const postcss = require("postcss");
const icssUtils = require("icss-utils");
const loaderUtils = require("loader-utils");

const pluginName = 'postcss-icss-find-imports';

module.exports = postcss.plugin(pluginName, (_options = {}) => {
	return function process(css, result) {
		let discoveredImports = new Set();

		let { icssImports, icssExports } = icssUtils.extractICSS(css, false);
		
		for (let importUrl of Object.keys(icssImports)) {
			discoveredImports.add(loaderUtils.parseString(importUrl));
		}

		for (let url of discoveredImports) {
			result.messages.push({
				pluginName: pluginName,
				type: "import",
				url: url
			});
		}
	};
});
